@@ -3,8 +3,7 @@
 from __future__ import print_function
 
 import operator
-import nose
-from itertools import product
+import pytest
 
 from pandas.compat import (zip, range, lrange, StringIO)
 from pandas import DataFrame, Series, Index, MultiIndex, date_range
@@ -15,11 +14,10 @@ from numpy.random import randn
 
 from pandas.util.testing import (assert_series_equal,
                                  assert_frame_equal,
-                                 assertRaises,
                                  makeCustomDataframe as mkdf)
 
 import pandas.util.testing as tm
-from pandas.computation import _NUMEXPR_INSTALLED
+from pandas.core.computation.check import _NUMEXPR_INSTALLED
 
 from pandas.tests.frame.common import TestData
 
@@ -28,21 +26,31 @@ PARSERS = 'python', 'pandas'
 ENGINES = 'python', 'numexpr'
 
 
+@pytest.fixture(params=PARSERS, ids=lambda x: x)
+def parser(request):
+    return request.param
+
+
+@pytest.fixture(params=ENGINES, ids=lambda x: x)
+def engine(request):
+    return request.param
+
+
 def skip_if_no_pandas_parser(parser):
     if parser != 'pandas':
-        raise nose.SkipTest("cannot evaluate with parser {0!r}".format(parser))
+        pytest.skip("cannot evaluate with parser {0!r}".format(parser))
 
 
 def skip_if_no_ne(engine='numexpr'):
     if engine == 'numexpr':
         if not _NUMEXPR_INSTALLED:
-            raise nose.SkipTest("cannot query engine numexpr when numexpr not "
-                                "installed")
+            pytest.skip("cannot query engine numexpr when numexpr not "
+                        "installed")
 
 
-class TestCompat(tm.TestCase):
+class TestCompat(object):
 
-    def setUp(self):
+    def setup_method(self, method):
         self.df = DataFrame({'A': [1, 2, 3]})
         self.expected1 = self.df[self.df.A > 0]
         self.expected2 = self.df.A + 1
@@ -82,15 +90,13 @@ class TestCompat(tm.TestCase):
             result = df.eval('A+1', engine='numexpr')
             assert_series_equal(result, self.expected2, check_names=False)
         else:
-            self.assertRaises(ImportError,
-                              lambda: df.query('A>0', engine='numexpr'))
-            self.assertRaises(ImportError,
-                              lambda: df.eval('A+1', engine='numexpr'))
+            pytest.raises(ImportError,
+                          lambda: df.query('A>0', engine='numexpr'))
+            pytest.raises(ImportError,
+                          lambda: df.eval('A+1', engine='numexpr'))
 
 
-class TestDataFrameEval(tm.TestCase, TestData):
-
-    _multiprocess_can_split_ = True
+class TestDataFrameEval(TestData):
 
     def test_ops(self):
 
@@ -141,19 +147,36 @@ class TestDataFrameEval(tm.TestCase, TestData):
         df = pd.DataFrame({'A': [1, 2, 3], 'B': ['a', 'b', 'b']})
 
         msg = "expr must be a string to be evaluated"
-        with tm.assertRaisesRegexp(ValueError, msg):
+        with tm.assert_raises_regex(ValueError, msg):
             df.query(lambda x: x.B == "b")
 
-        with tm.assertRaisesRegexp(ValueError, msg):
+        with tm.assert_raises_regex(ValueError, msg):
             df.query(111)
 
+    def test_query_empty_string(self):
+        # GH 13139
+        df = pd.DataFrame({'A': [1, 2, 3]})
 
-class TestDataFrameQueryWithMultiIndex(tm.TestCase):
+        msg = "expr cannot be an empty string"
+        with tm.assert_raises_regex(ValueError, msg):
+            df.query('')
 
-    _multiprocess_can_split_ = True
+    def test_eval_resolvers_as_list(self):
+        # GH 14095
+        df = DataFrame(randn(10, 2), columns=list('ab'))
+        dict1 = {'a': 1}
+        dict2 = {'b': 2}
+        assert (df.eval('a + b', resolvers=[dict1, dict2]) ==
+                dict1['a'] + dict2['b'])
+        assert (pd.eval('a + b', resolvers=[dict1, dict2]) ==
+                dict1['a'] + dict2['b'])
 
-    def check_query_with_named_multiindex(self, parser, engine):
+
+class TestDataFrameQueryWithMultiIndex(object):
+
+    def test_query_with_named_multiindex(self, parser, engine):
         tm.skip_if_no_ne(engine)
+        skip_if_no_pandas_parser(parser)
         a = np.random.choice(['red', 'green'], size=10)
         b = np.random.choice(['eggs', 'ham'], size=10)
         index = MultiIndex.from_arrays([a, b], names=['color', 'food'])
@@ -201,12 +224,9 @@ class TestDataFrameQueryWithMultiIndex(tm.TestCase):
         assert_frame_equal(res1, exp)
         assert_frame_equal(res2, exp)
 
-    def test_query_with_named_multiindex(self):
-        for parser, engine in product(['pandas'], ENGINES):
-            yield self.check_query_with_named_multiindex, parser, engine
-
-    def check_query_with_unnamed_multiindex(self, parser, engine):
+    def test_query_with_unnamed_multiindex(self, parser, engine):
         tm.skip_if_no_ne(engine)
+        skip_if_no_pandas_parser(parser)
         a = np.random.choice(['red', 'green'], size=10)
         b = np.random.choice(['eggs', 'ham'], size=10)
         index = MultiIndex.from_arrays([a, b])
@@ -295,12 +315,9 @@ class TestDataFrameQueryWithMultiIndex(tm.TestCase):
         assert_frame_equal(res1, exp)
         assert_frame_equal(res2, exp)
 
-    def test_query_with_unnamed_multiindex(self):
-        for parser, engine in product(['pandas'], ENGINES):
-            yield self.check_query_with_unnamed_multiindex, parser, engine
-
-    def check_query_with_partially_named_multiindex(self, parser, engine):
+    def test_query_with_partially_named_multiindex(self, parser, engine):
         tm.skip_if_no_ne(engine)
+        skip_if_no_pandas_parser(parser)
         a = np.random.choice(['red', 'green'], size=10)
         b = np.arange(10)
         index = MultiIndex.from_arrays([a, b])
@@ -328,17 +345,7 @@ class TestDataFrameQueryWithMultiIndex(tm.TestCase):
         exp = df[ind != "red"]
         assert_frame_equal(res, exp)
 
-    def test_query_with_partially_named_multiindex(self):
-        for parser, engine in product(['pandas'], ENGINES):
-            yield (self.check_query_with_partially_named_multiindex,
-                   parser, engine)
-
     def test_query_multiindex_get_index_resolvers(self):
-        for parser, engine in product(['pandas'], ENGINES):
-            yield (self.check_query_multiindex_get_index_resolvers, parser,
-                   engine)
-
-    def check_query_multiindex_get_index_resolvers(self, parser, engine):
         df = mkdf(10, 3, r_idx_nlevels=2, r_idx_names=['spam', 'eggs'])
         resolvers = df._get_index_resolvers()
 
@@ -362,41 +369,31 @@ class TestDataFrameQueryWithMultiIndex(tm.TestCase):
             else:
                 raise AssertionError("object must be a Series or Index")
 
-    def test_raise_on_panel_with_multiindex(self):
-        for parser, engine in product(PARSERS, ENGINES):
-            yield self.check_raise_on_panel_with_multiindex, parser, engine
-
-    def check_raise_on_panel_with_multiindex(self, parser, engine):
+    def test_raise_on_panel_with_multiindex(self, parser, engine):
         tm.skip_if_no_ne()
         p = tm.makePanel(7)
         p.items = tm.makeCustomIndex(len(p.items), nlevels=2)
-        with tm.assertRaises(NotImplementedError):
+        with pytest.raises(NotImplementedError):
             pd.eval('p + 1', parser=parser, engine=engine)
 
-    def test_raise_on_panel4d_with_multiindex(self):
-        for parser, engine in product(PARSERS, ENGINES):
-            yield self.check_raise_on_panel4d_with_multiindex, parser, engine
-
-    def check_raise_on_panel4d_with_multiindex(self, parser, engine):
+    def test_raise_on_panel4d_with_multiindex(self, parser, engine):
         tm.skip_if_no_ne()
         p4d = tm.makePanel4D(7)
         p4d.items = tm.makeCustomIndex(len(p4d.items), nlevels=2)
-        with tm.assertRaises(NotImplementedError):
+        with pytest.raises(NotImplementedError):
             pd.eval('p4d + 1', parser=parser, engine=engine)
 
 
-class TestDataFrameQueryNumExprPandas(tm.TestCase):
+class TestDataFrameQueryNumExprPandas(object):
 
     @classmethod
-    def setUpClass(cls):
-        super(TestDataFrameQueryNumExprPandas, cls).setUpClass()
+    def setup_class(cls):
         cls.engine = 'numexpr'
         cls.parser = 'pandas'
         tm.skip_if_no_ne(cls.engine)
 
     @classmethod
-    def tearDownClass(cls):
-        super(TestDataFrameQueryNumExprPandas, cls).tearDownClass()
+    def teardown_class(cls):
         del cls.engine, cls.parser
 
     def test_date_query_with_attribute_access(self):
@@ -470,7 +467,7 @@ class TestDataFrameQueryNumExprPandas(tm.TestCase):
         df = DataFrame(d)
         df.loc[np.random.rand(n) > 0.5, 'dates1'] = pd.NaT
         df.set_index('dates1', inplace=True, drop=True)
-        res = df.query('index < 20130101 < dates3', engine=engine,
+        res = df.query('dates1 < 20130101 < dates3', engine=engine,
                        parser=parser)
         expec = df[(df.index.to_series() < '20130101') &
                    ('20130101' < df.dates3)]
@@ -486,18 +483,18 @@ class TestDataFrameQueryNumExprPandas(tm.TestCase):
         ops = '==', '!=', '<', '>', '<=', '>='
 
         for op in ops:
-            with tm.assertRaises(TypeError):
+            with pytest.raises(TypeError):
                 df.query('dates %s nondate' % op, parser=parser, engine=engine)
 
     def test_query_syntax_error(self):
         engine, parser = self.engine, self.parser
         df = DataFrame({"i": lrange(10), "+": lrange(3, 13),
                         "r": lrange(4, 14)})
-        with tm.assertRaises(SyntaxError):
+        with pytest.raises(SyntaxError):
             df.query('i - +', engine=engine, parser=parser)
 
     def test_query_scope(self):
-        from pandas.computation.ops import UndefinedVariableError
+        from pandas.core.computation.ops import UndefinedVariableError
         engine, parser = self.engine, self.parser
         skip_if_no_pandas_parser(parser)
 
@@ -513,34 +510,34 @@ class TestDataFrameQueryNumExprPandas(tm.TestCase):
         assert_frame_equal(res, expected)
 
         # no local variable c
-        with tm.assertRaises(UndefinedVariableError):
+        with pytest.raises(UndefinedVariableError):
             df.query('@a > b > @c', engine=engine, parser=parser)
 
         # no column named 'c'
-        with tm.assertRaises(UndefinedVariableError):
+        with pytest.raises(UndefinedVariableError):
             df.query('@a > b > c', engine=engine, parser=parser)
 
     def test_query_doesnt_pickup_local(self):
-        from pandas.computation.ops import UndefinedVariableError
+        from pandas.core.computation.ops import UndefinedVariableError
 
         engine, parser = self.engine, self.parser
         n = m = 10
         df = DataFrame(np.random.randint(m, size=(n, 3)), columns=list('abc'))
 
         # we don't pick up the local 'sin'
-        with tm.assertRaises(UndefinedVariableError):
+        with pytest.raises(UndefinedVariableError):
             df.query('sin > 5', engine=engine, parser=parser)
 
     def test_query_builtin(self):
-        from pandas.computation.engines import NumExprClobberingError
+        from pandas.core.computation.engines import NumExprClobberingError
         engine, parser = self.engine, self.parser
 
         n = m = 10
         df = DataFrame(np.random.randint(m, size=(n, 3)), columns=list('abc'))
 
         df.index.name = 'sin'
-        with tm.assertRaisesRegexp(NumExprClobberingError,
-                                   'Variables in expression.+'):
+        with tm.assert_raises_regex(NumExprClobberingError,
+                                    'Variables in expression.+'):
             df.query('sin > 5', engine=engine, parser=parser)
 
     def test_query(self):
@@ -610,12 +607,12 @@ class TestDataFrameQueryNumExprPandas(tm.TestCase):
         assert_frame_equal(result, expected)
 
     def test_nested_raises_on_local_self_reference(self):
-        from pandas.computation.ops import UndefinedVariableError
+        from pandas.core.computation.ops import UndefinedVariableError
 
         df = DataFrame(np.random.randn(5, 3))
 
         # can't reference ourself b/c we're a local so @ is necessary
-        with tm.assertRaises(UndefinedVariableError):
+        with pytest.raises(UndefinedVariableError):
             df.query('df > 0', engine=self.engine, parser=self.parser)
 
     def test_local_syntax(self):
@@ -669,12 +666,12 @@ class TestDataFrameQueryNumExprPandas(tm.TestCase):
         assert_frame_equal(result, expected)
 
     def test_query_undefined_local(self):
-        from pandas.computation.ops import UndefinedVariableError
+        from pandas.core.computation.ops import UndefinedVariableError
         engine, parser = self.engine, self.parser
         skip_if_no_pandas_parser(parser)
         df = DataFrame(np.random.rand(10, 2), columns=list('ab'))
-        with tm.assertRaisesRegexp(UndefinedVariableError,
-                                   "local variable 'c' is not defined"):
+        with tm.assert_raises_regex(UndefinedVariableError,
+                                    "local variable 'c' is not defined"):
             df.query('a == @c', engine=engine, parser=parser)
 
     def test_index_resolvers_come_after_columns_with_the_same_name(self):
@@ -720,8 +717,8 @@ class TestDataFrameQueryNumExprPandas(tm.TestCase):
 class TestDataFrameQueryNumExprPython(TestDataFrameQueryNumExprPandas):
 
     @classmethod
-    def setUpClass(cls):
-        super(TestDataFrameQueryNumExprPython, cls).setUpClass()
+    def setup_class(cls):
+        super(TestDataFrameQueryNumExprPython, cls).setup_class()
         cls.engine = 'numexpr'
         cls.parser = 'python'
         tm.skip_if_no_ne(cls.engine)
@@ -785,26 +782,26 @@ class TestDataFrameQueryNumExprPython(TestDataFrameQueryNumExprPandas):
         df['dates3'] = date_range('1/1/2014', periods=n)
         df.loc[np.random.rand(n) > 0.5, 'dates1'] = pd.NaT
         df.set_index('dates1', inplace=True, drop=True)
-        with tm.assertRaises(NotImplementedError):
+        with pytest.raises(NotImplementedError):
             df.query('index < 20130101 < dates3', engine=engine, parser=parser)
 
     def test_nested_scope(self):
-        from pandas.computation.ops import UndefinedVariableError
+        from pandas.core.computation.ops import UndefinedVariableError
         engine = self.engine
         parser = self.parser
         # smoke test
         x = 1  # noqa
         result = pd.eval('x + 1', engine=engine, parser=parser)
-        self.assertEqual(result, 2)
+        assert result == 2
 
         df = DataFrame(np.random.randn(5, 3))
         df2 = DataFrame(np.random.randn(5, 3))
 
         # don't have the pandas parser
-        with tm.assertRaises(SyntaxError):
+        with pytest.raises(SyntaxError):
             df.query('(@df>0) & (@df2>0)', engine=engine, parser=parser)
 
-        with tm.assertRaises(UndefinedVariableError):
+        with pytest.raises(UndefinedVariableError):
             df.query('(df>0) & (df2>0)', engine=engine, parser=parser)
 
         expected = df[(df > 0) & (df2 > 0)]
@@ -821,8 +818,8 @@ class TestDataFrameQueryNumExprPython(TestDataFrameQueryNumExprPandas):
 class TestDataFrameQueryPythonPandas(TestDataFrameQueryNumExprPandas):
 
     @classmethod
-    def setUpClass(cls):
-        super(TestDataFrameQueryPythonPandas, cls).setUpClass()
+    def setup_class(cls):
+        super(TestDataFrameQueryPythonPandas, cls).setup_class()
         cls.engine = 'python'
         cls.parser = 'pandas'
         cls.frame = TestData().frame
@@ -842,8 +839,8 @@ class TestDataFrameQueryPythonPandas(TestDataFrameQueryNumExprPandas):
 class TestDataFrameQueryPythonPython(TestDataFrameQueryNumExprPython):
 
     @classmethod
-    def setUpClass(cls):
-        super(TestDataFrameQueryPythonPython, cls).setUpClass()
+    def setup_class(cls):
+        super(TestDataFrameQueryPythonPython, cls).setup_class()
         cls.engine = cls.parser = 'python'
         cls.frame = TestData().frame
 
@@ -859,9 +856,9 @@ class TestDataFrameQueryPythonPython(TestDataFrameQueryNumExprPython):
         assert_frame_equal(expected, result)
 
 
-class TestDataFrameQueryStrings(tm.TestCase):
+class TestDataFrameQueryStrings(object):
 
-    def check_str_query_method(self, parser, engine):
+    def test_str_query_method(self, parser, engine):
         tm.skip_if_no_ne(engine)
         df = DataFrame(randn(10, 1), columns=['b'])
         df['strings'] = Series(list('aabbccddee'))
@@ -879,8 +876,9 @@ class TestDataFrameQueryStrings(tm.TestCase):
 
             for lhs, op, rhs in zip(lhs, ops, rhs):
                 ex = '{lhs} {op} {rhs}'.format(lhs=lhs, op=op, rhs=rhs)
-                assertRaises(NotImplementedError, df.query, ex, engine=engine,
-                             parser=parser, local_dict={'strings': df.strings})
+                pytest.raises(NotImplementedError, df.query, ex,
+                              engine=engine, parser=parser,
+                              local_dict={'strings': df.strings})
         else:
             res = df.query('"a" == strings', engine=engine, parser=parser)
             assert_frame_equal(res, expect)
@@ -897,15 +895,7 @@ class TestDataFrameQueryStrings(tm.TestCase):
             assert_frame_equal(res, expect)
             assert_frame_equal(res, df[~df.strings.isin(['a'])])
 
-    def test_str_query_method(self):
-        for parser, engine in product(PARSERS, ENGINES):
-            yield self.check_str_query_method, parser, engine
-
-    def test_str_list_query_method(self):
-        for parser, engine in product(PARSERS, ENGINES):
-            yield self.check_str_list_query_method, parser, engine
-
-    def check_str_list_query_method(self, parser, engine):
+    def test_str_list_query_method(self, parser, engine):
         tm.skip_if_no_ne(engine)
         df = DataFrame(randn(10, 1), columns=['b'])
         df['strings'] = Series(list('aabbccddee'))
@@ -923,7 +913,7 @@ class TestDataFrameQueryStrings(tm.TestCase):
 
             for lhs, op, rhs in zip(lhs, ops, rhs):
                 ex = '{lhs} {op} {rhs}'.format(lhs=lhs, op=op, rhs=rhs)
-                with tm.assertRaises(NotImplementedError):
+                with pytest.raises(NotImplementedError):
                     df.query(ex, engine=engine, parser=parser)
         else:
             res = df.query('strings == ["a", "b"]', engine=engine,
@@ -944,7 +934,7 @@ class TestDataFrameQueryStrings(tm.TestCase):
                            parser=parser)
             assert_frame_equal(res, expect)
 
-    def check_query_with_string_columns(self, parser, engine):
+    def test_query_with_string_columns(self, parser, engine):
         tm.skip_if_no_ne(engine)
         df = DataFrame({'a': list('aaaabbbbcccc'),
                         'b': list('aabbccddeeff'),
@@ -959,17 +949,13 @@ class TestDataFrameQueryStrings(tm.TestCase):
             expec = df[df.a.isin(df.b) & (df.c < df.d)]
             assert_frame_equal(res, expec)
         else:
-            with assertRaises(NotImplementedError):
+            with pytest.raises(NotImplementedError):
                 df.query('a in b', parser=parser, engine=engine)
 
-            with assertRaises(NotImplementedError):
+            with pytest.raises(NotImplementedError):
                 df.query('a in b and c < d', parser=parser, engine=engine)
 
-    def test_query_with_string_columns(self):
-        for parser, engine in product(PARSERS, ENGINES):
-            yield self.check_query_with_string_columns, parser, engine
-
-    def check_object_array_eq_ne(self, parser, engine):
+    def test_object_array_eq_ne(self, parser, engine):
         tm.skip_if_no_ne(engine)
         df = DataFrame({'a': list('aaaabbbbcccc'),
                         'b': list('aabbccddeeff'),
@@ -983,11 +969,7 @@ class TestDataFrameQueryStrings(tm.TestCase):
         exp = df[df.a != df.b]
         assert_frame_equal(res, exp)
 
-    def test_object_array_eq_ne(self):
-        for parser, engine in product(PARSERS, ENGINES):
-            yield self.check_object_array_eq_ne, parser, engine
-
-    def check_query_with_nested_strings(self, parser, engine):
+    def test_query_with_nested_strings(self, parser, engine):
         tm.skip_if_no_ne(engine)
         skip_if_no_pandas_parser(parser)
         raw = """id          event          timestamp
@@ -1011,11 +993,7 @@ class TestDataFrameQueryStrings(tm.TestCase):
                        engine=engine)
         assert_frame_equal(expected, res)
 
-    def test_query_with_nested_string(self):
-        for parser, engine in product(PARSERS, ENGINES):
-            yield self.check_query_with_nested_strings, parser, engine
-
-    def check_query_with_nested_special_character(self, parser, engine):
+    def test_query_with_nested_special_character(self, parser, engine):
         skip_if_no_pandas_parser(parser)
         tm.skip_if_no_ne(engine)
         df = DataFrame({'a': ['a', 'b', 'test & test'],
@@ -1024,12 +1002,7 @@ class TestDataFrameQueryStrings(tm.TestCase):
         expec = df[df.a == 'test & test']
         assert_frame_equal(res, expec)
 
-    def test_query_with_nested_special_character(self):
-        for parser, engine in product(PARSERS, ENGINES):
-            yield (self.check_query_with_nested_special_character,
-                   parser, engine)
-
-    def check_query_lex_compare_strings(self, parser, engine):
+    def test_query_lex_compare_strings(self, parser, engine):
         tm.skip_if_no_ne(engine=engine)
         import operator as opr
 
@@ -1044,11 +1017,7 @@ class TestDataFrameQueryStrings(tm.TestCase):
             expected = df[func(df.X, 'd')]
             assert_frame_equal(res, expected)
 
-    def test_query_lex_compare_strings(self):
-        for parser, engine in product(PARSERS, ENGINES):
-            yield self.check_query_lex_compare_strings, parser, engine
-
-    def check_query_single_element_booleans(self, parser, engine):
+    def test_query_single_element_booleans(self, parser, engine):
         tm.skip_if_no_ne(engine)
         columns = 'bid', 'bidsize', 'ask', 'asksize'
         data = np.random.randint(2, size=(1, len(columns))).astype(bool)
@@ -1057,12 +1026,9 @@ class TestDataFrameQueryStrings(tm.TestCase):
         expected = df[df.bid & df.ask]
         assert_frame_equal(res, expected)
 
-    def test_query_single_element_booleans(self):
-        for parser, engine in product(PARSERS, ENGINES):
-            yield self.check_query_single_element_booleans, parser, engine
-
-    def check_query_string_scalar_variable(self, parser, engine):
+    def test_query_string_scalar_variable(self, parser, engine):
         tm.skip_if_no_ne(engine)
+        skip_if_no_pandas_parser(parser)
         df = pd.DataFrame({'Symbol': ['BUD US', 'BUD US', 'IBM US', 'IBM US'],
                            'Price': [109.70, 109.72, 183.30, 183.35]})
         e = df[df.Symbol == 'BUD US']
@@ -1070,24 +1036,19 @@ class TestDataFrameQueryStrings(tm.TestCase):
         r = df.query('Symbol == @symb', parser=parser, engine=engine)
         assert_frame_equal(e, r)
 
-    def test_query_string_scalar_variable(self):
-        for parser, engine in product(['pandas'], ENGINES):
-            yield self.check_query_string_scalar_variable, parser, engine
 
-
-class TestDataFrameEvalNumExprPandas(tm.TestCase):
+class TestDataFrameEvalNumExprPandas(object):
 
     @classmethod
-    def setUpClass(cls):
-        super(TestDataFrameEvalNumExprPandas, cls).setUpClass()
+    def setup_class(cls):
         cls.engine = 'numexpr'
         cls.parser = 'pandas'
         tm.skip_if_no_ne()
 
-    def setUp(self):
+    def setup_method(self, method):
         self.frame = DataFrame(randn(10, 3), columns=list('abc'))
 
-    def tearDown(self):
+    def teardown_method(self, method):
         del self.frame
 
     def test_simple_expr(self):
@@ -1105,9 +1066,9 @@ class TestDataFrameEvalNumExprPandas(tm.TestCase):
         df = DataFrame({'a': [1, 2], 'b': ['c', 'd']})
         ops = '+', '-', '*', '/'
         for op in ops:
-            with tm.assertRaisesRegexp(TypeError,
-                                       "unsupported operand type\(s\) for "
-                                       ".+: '.+' and '.+'"):
+            with tm.assert_raises_regex(TypeError,
+                                        "unsupported operand type\(s\) "
+                                        "for .+: '.+' and '.+'"):
                 df.eval('a {0} b'.format(op), engine=self.engine,
                         parser=self.parser)
 
@@ -1115,8 +1076,8 @@ class TestDataFrameEvalNumExprPandas(tm.TestCase):
 class TestDataFrameEvalNumExprPython(TestDataFrameEvalNumExprPandas):
 
     @classmethod
-    def setUpClass(cls):
-        super(TestDataFrameEvalNumExprPython, cls).setUpClass()
+    def setup_class(cls):
+        super(TestDataFrameEvalNumExprPython, cls).setup_class()
         cls.engine = 'numexpr'
         cls.parser = 'python'
         tm.skip_if_no_ne(cls.engine)
@@ -1125,8 +1086,8 @@ class TestDataFrameEvalNumExprPython(TestDataFrameEvalNumExprPandas):
 class TestDataFrameEvalPythonPandas(TestDataFrameEvalNumExprPandas):
 
     @classmethod
-    def setUpClass(cls):
-        super(TestDataFrameEvalPythonPandas, cls).setUpClass()
+    def setup_class(cls):
+        super(TestDataFrameEvalPythonPandas, cls).setup_class()
         cls.engine = 'python'
         cls.parser = 'pandas'
 
@@ -1134,11 +1095,5 @@ class TestDataFrameEvalPythonPandas(TestDataFrameEvalNumExprPandas):
 class TestDataFrameEvalPythonPython(TestDataFrameEvalNumExprPython):
 
     @classmethod
-    def setUpClass(cls):
-        super(TestDataFrameEvalPythonPython, cls).tearDownClass()
+    def setup_class(cls):
         cls.engine = cls.parser = 'python'
-
-
-if __name__ == '__main__':
-    nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
-                   exit=False)
